@@ -1,24 +1,58 @@
+#' Position (size): description - example value
+#' 0 (8): determine endianness - 1.0 (double)
+#' 8 (4 x 3): version number - [1, 0, 0] (int[3])
+#' 20 (4): SEXP type - 14L (SEXPTYPE, data are double type)
+#' 24 (4): element size - 8L (int, data element size is 8)
+#' 28 (8): number of last dimensions per partition - 1 (double)
+#' 36 (8): total number of elements - 24 (double)
+#' 44 (4): number of dimensions (ndims) - 3 (int)
+#' 48 (8 x ndims): dimensions - [2, 3, 4] (double)
+#' 
+#' ...
+#' 
+#' 1012 (4): size of the header - 72 (int)
+#' 1016 (8): number of elements in the file - (double)
+#' 
+#' @noRd
+NULL
+
 write_header <- function(fid, partition, dimension, type, size){
     seek(con = fid, where = 0, rw = "write")
+    
+    # 0 (8): determine endianness - 1.0 (double)
     writeBin(con = fid, object = 1.0, size = 8L, endian = ENDIANNESS)
+    
+    # 8 (4 x 3): version number - [1, 0, 0] (int[3])
     writeBin(con = fid, object = FILE_VER, size = 4L, endian = ENDIANNESS)
     
     sexp_type <- type_to_sexp(type)
+    
+    # 20 (4): SEXP type - 14L (SEXPTYPE, data are double type)
     writeBin(con = fid, object = sexp_type, size = 4L,
              endian = ENDIANNESS)
+    
+    # 24 (4): element size - 8L (int, data element size is 8)
     writeBin(con = fid, object = as.integer(size), size = 4L,
              endian = ENDIANNESS)
     
     # partition number
     partition <- as.double(partition)
+    
+    # 28 (8): number of last dimensions per partition - 1 (double)
     writeBin(con = fid, object = partition, size = 8L, endian = ENDIANNESS)
     
-    # write length of dimension
+    ## # write length of dimension
     dimension <- as.double(dimension)
     len <- prod(dimension)
+    
+    # 36 (8): total number of elements - 24 (double)
     writeBin(con = fid, object = len, size = 8L, endian = ENDIANNESS)
+    
+    # 44 (4): number of dimensions (ndims) - 3 (int)
     writeBin(con = fid, object = length(dimension), size = 4L, 
              endian = ENDIANNESS)
+    
+    # 48 (8 x ndims): dimensions - [2, 3, 4] (double)
     writeBin(con = fid, object = dimension, size = 8L, 
              endian = ENDIANNESS)
     header_len <- 48L + 8L * length(dimension)
