@@ -11,12 +11,15 @@
 #' @param dimnames whether to preserve \code{\link[base]{dimnames}}
 #' @param value value to substitute or set
 #' @param na.rm whether to remove \code{NA} values during the calculation
+#' @param split_dim internally used; split dimension and calculate indices to
+#' manually speed up the subset; value ranged from 0 to size of dimension minus
+#' one.
 #' @param i,... index set, or passed to other methods
 NULL
 
 #' @describeIn S3-filearray get element by position
 #' @export
-`[.FileArray` <- function(x, ..., drop = TRUE, reshape = NULL, strict = TRUE, dimnames = TRUE) {
+`[.FileArray` <- function(x, ..., drop = TRUE, reshape = NULL, strict = TRUE, dimnames = TRUE, split_dim = 0) {
     if(!x$valid()){
         stop("Invalid file array")
     }
@@ -65,7 +68,7 @@ NULL
     }
     
     # guess split dim
-    max_buffer <- max_buffer_size() / elem_size
+    max_buffer <- get_buffer_size() / elem_size
     
     if(length(listOrEnv) == length(dim)){
         idxrange <- sapply(listOrEnv, function(x){
@@ -76,21 +79,24 @@ NULL
     } else {
         idxrange <- dim
     }
-    # worst-case time-complexity
-    time_complexity <-
-        sapply(seq_len(length(dim) - 1), function(split_dim) {
-            dim[[length(dim)]] <- 1
-            idx1dim <- dim[seq_len(split_dim)]
-            idx1dim[[split_dim]] <- idxrange[[split_dim]]
-            idx1len <- prod(idx1dim)
-            idx2len <- prod(dim[-seq_len(split_dim)])
-            buffer_sz <-
-                ifelse(idx1len > max_buffer, max_buffer, idx1len)
-            nloops <- ceiling(idx1len / buffer_sz)
-            (idx1len * nloops + idx2len) * idx2len
-        })
-    split_dim <- which.min(time_complexity)
-    split_dim <- split_dim[[length(split_dim)]]
+    split_dim <- as.integer(split_dim)
+    if(is.na(split_dim) || split_dim <= 0 || split_dim >= length(dim)){
+        # worst-case time-complexity
+        time_complexity <-
+            sapply(seq_len(length(dim) - 1), function(split_dim) {
+                dim[[length(dim)]] <- 1
+                idx1dim <- dim[seq_len(split_dim)]
+                idx1dim[[split_dim]] <- idxrange[[split_dim]]
+                idx1len <- prod(idx1dim)
+                idx2len <- prod(dim[-seq_len(split_dim)])
+                buffer_sz <-
+                    ifelse(idx1len > max_buffer, max_buffer, idx1len)
+                nloops <- ceiling(idx1len / buffer_sz)
+                (idx1len * nloops + idx2len) * idx2len
+            })
+        split_dim <- which.min(time_complexity)
+        split_dim <- split_dim[[length(split_dim)]]
+    }
     
 
     FARR_subset2(
@@ -99,7 +105,7 @@ NULL
         reshape = reshape,
         drop = drop,
         use_dimnames = isTRUE(dimnames),
-        thread_buffer = max_buffer_size(), 
+        thread_buffer = get_buffer_size(), 
         split_dim = split_dim,
         strict = isTRUE(strict)
     )
