@@ -34,242 +34,6 @@ int get_buffer_nelem(SEXPTYPE type){
  * Subset - internal (multithread here)
  ***********************************************************/
 
-
-template <typename T, typename B>
-bool FARR_subset_template(
-        const std::string& filebase, 
-        const List& sch,
-        T* ret_ptr, const T na, const R_xlen_t& retlen,
-        void (*transform)(const B*, T*, const bool&)
-){
-    
-    SEXP idx1 = sch["idx1"];
-    SEXP idx1range = sch["idx1range"];
-    List idx2s = sch["idx2s"];
-    int64_t block_size = (int64_t) (sch["block_size"]);
-    IntegerVector partitions = sch["partitions"];
-    IntegerVector idx2lens = sch["idx2lens"];
-    
-    R_xlen_t niter = partitions.length();
-    R_xlen_t idx1len = Rf_xlength(idx1);
-    
-    int64_t* idx1rangeptr = (int64_t*) REAL(idx1range);
-    int64_t idx1_start = *idx1rangeptr, idx1_end = *(idx1rangeptr + 1);
-    
-    if( idx1_start == NA_INTEGER64 || idx1_end < 0 || idx1_start < 0 ){
-        // idx1 are all NAs, no need to subset, return NA
-        
-        T* retptr = ret_ptr;
-        for(R_xlen_t jj = 0; jj < retlen; jj++){
-            *retptr++ = na;
-        }
-        return(false);
-    }
-    
-    int err = -1;
-    std::string error_msg = "";
-    // char* buffer[nbuffers];
-    
-    int ncores = getThreads();
-    if(ncores > niter){
-        ncores = niter;
-    }
-    const boost::interprocess::mode_t mode = boost::interprocess::read_only;
-    const int elem_size = sizeof(B);
-    const bool swap_endian = !isLittleEndian();
-    
-#pragma omp parallel num_threads(ncores) 
-{
-#pragma omp for schedule(static, 1) nowait
-    for(R_xlen_t ii = 0; ii < niter; ii++){
-        
-        int part = partitions[ii];
-        int64_t skips = 0;
-        if(ii > 0){
-            skips = idx2lens[ii - 1];
-        }
-        int64_t idx2len = idx2lens[ii] - skips;
-        
-        // TODO: change
-        T* retptr = ret_ptr + skips * idx1len;
-        for(R_xlen_t jj = 0; jj < idx2len * idx1len; jj++, retptr++ ){
-            *retptr = na;
-        }
-        
-        // TODO: change
-        retptr = ret_ptr + skips * idx1len;
-        
-        SEXP idx2 = idx2s[ii];
-        int64_t idx2_start = NA_INTEGER64, idx2_end = -1;
-        int64_t* ptr2 = (int64_t*) REAL(idx1); 
-        for(ptr2 = (int64_t*) REAL(idx2); idx2len > 0; idx2len--, ptr2++ ){
-            if( *ptr2 == NA_INTEGER64 ){
-                continue;
-            }
-            if( *ptr2 < idx2_start || idx2_start == NA_INTEGER64 ){
-                idx2_start = *ptr2;
-            }
-            if( idx2_end < *ptr2 ){
-                idx2_end = *ptr2;
-            }
-        }
-        
-        if( idx2_start == NA_INTEGER64 || idx2_end < 0 || idx2_start < 0 ){
-            // This is NA partition, no need to subset
-            continue;
-        }
-        
-        const int idx2_sorted = kinda_sorted(idx2, idx2_start, 1);
-        std::string file = filebase + std::to_string(part) + ".farr";
-        
-        try{
-            boost::interprocess::file_mapping fm(file.c_str(), mode);
-            boost::interprocess::mapped_region region(
-                    fm, mode, 
-                    FARR_HEADER_LENGTH + elem_size * (
-                            idx2_start * block_size + idx1_start
-                    ), 
-                    elem_size * (
-                            idx1_end - idx1_start + 1 +
-                                block_size * (idx2_end - idx2_start)
-                    ));
-            region.advise(boost::interprocess::mapped_region::advice_sequential);
-            const B* mmap_ptr = static_cast<const B*>(region.get_address());
-            const int64_t content_size = region.get_size() / elem_size;
-            
-            
-            // prepare for all the pointers, local variables
-            int64_t* idx2ptr = INTEGER64(idx2);
-            R_xlen_t idx2len = Rf_xlength(idx2);
-            R_xlen_t ii_idx2 = 0;
-            int64_t start_idx = idx1_start;
-
-            int64_t* idx1ptr = INTEGER64(idx1);
-            
-            R_xlen_t jj = 0;
-            T* retptr2 = retptr;
-            
-            for(; ii_idx2 < idx2len; ii_idx2++, idx2ptr++) {
-                
-                if ( *idx2ptr == NA_INTEGER64 ){
-                    continue;
-                }
-                
-                // Rcout << block << "\n";
-                
-                // read current block!
-                retptr2 = retptr + ii_idx2 * idx1len;
-                start_idx = block_size * (*idx2ptr - idx2_start);
-                
-                if( start_idx >= content_size ){
-                    if( idx2_sorted ){
-                        break;
-                    }
-                    continue;
-                }
-                
-                idx1ptr = INTEGER64(idx1);
-                
-                for(jj = 0; jj < idx1len; jj++, idx1ptr++) {
-                    if(*idx1ptr == NA_INTEGER64){ continue; }
-                    
-                    transform(mmap_ptr + (start_idx + *idx1ptr - idx1_start), retptr2 + jj, swap_endian);
-                }
-                
-            }
-        } catch(...) {
-            // Debug use
-            // err = part;
-        }
-        
-    }
-}
-
-    if( err >= 0 ){
-        stop("Error while trying to read partition "+std::to_string(err+1)+
-            ". Reason: " + error_msg);
-        // Rcout << error_msg << "\n";
-    }
-    return(true);
-}
-
-SEXP FARR_subset(const std::string& filebase, 
-                 const List& sch,
-                 const SEXPTYPE type,
-                 SEXP ret){
-    std::string fbase = correct_filebase(filebase);
-    
-    // SEXP idx1 = sch["idx1"];
-    // IntegerVector partitions = sch["partitions"];
-    // IntegerVector idx2lens = sch["idx2lens"];
-    
-    // R_xlen_t niter = partitions.length();
-    
-    // R_xlen_t idx1len = Rf_xlength(idx1);
-    // R_xlen_t retlen = idx1len * idx2lens[niter - 1];
-    // 
-    // SEXPTYPE ret_type = array_memory_sxptype(type);
-    // SEXP ret = PROTECT(Rf_allocVector(ret_type, retlen));
-    
-    R_xlen_t retlen = Rf_xlength(ret);
-    SEXP result_dim = sch["result_dim"];
-    Rf_setAttrib(ret, R_DimSymbol, result_dim);
-    
-    // SEXPTYPE buffer_type = file_buffer_sxptype(type);
-    // int buffer_nelems = get_buffer_nelem(type);
-    int ncores = getThreads();
-    if( ncores < 1 ){
-        stop("Thread number and buffer pool size must be positive.");
-    }
-    
-    switch(type){
-    case INTSXP: {
-        FARR_subset_template<int, int>(
-            fbase, sch, INTEGER(ret), NA_INTEGER, retlen,
-            &transform_asis);
-        break;
-    }
-    case REALSXP: {
-        FARR_subset_template<double, double>(
-            fbase, sch, REAL(ret), NA_REAL, retlen,
-            &transform_asis);
-        break;
-    }
-    case FLTSXP: {
-        FARR_subset_template<double, float>(
-            fbase, sch, REAL(ret), NA_REAL, retlen,
-            &transform_float);
-        break;
-    }
-    case RAWSXP: {
-        FARR_subset_template<Rbyte, Rbyte>(
-            fbase, sch, RAW(ret), NA_RBYTE, retlen,
-            &transform_asis);
-        break;
-    }
-    case LGLSXP: {
-        FARR_subset_template<int, Rbyte>(
-            fbase, sch, LOGICAL(ret), NA_LOGICAL, retlen,
-            &transform_logical);
-        break;
-    }
-    case CPLXSXP: {
-        na_cplx_dbl();
-        Rcomplex na_cplx;
-        na_cplx.i = NA_REAL;
-        na_cplx.r = NA_REAL;
-        FARR_subset_template<Rcomplex, double>(
-            fbase, sch, COMPLEX(ret), na_cplx, retlen,
-            &transform_complex);
-        break;
-    }
-    default:
-        stop("Unsupported SEXP type");
-    }
-    
-    return(ret);
-}
-
 // [[Rcpp::export]]
 SEXP FARR_subset_sequential(
         const std::string& filebase, 
@@ -411,6 +175,483 @@ SEXP FARR_subset_sequential(
     
 }
 
+template <typename T, typename B>
+bool FARR_subset_mmap(
+        const std::string& filebase, 
+        const List& sch,
+        T* ret_ptr, const T na, const R_xlen_t& retlen,
+        void (*transform)(const B*, T*, const bool&)
+){
+    
+    SEXP idx1 = sch["idx1"];
+    SEXP idx1range = sch["idx1range"];
+    List idx2s = sch["idx2s"];
+    int64_t block_size = (int64_t) (sch["block_size"]);
+    IntegerVector partitions = sch["partitions"];
+    IntegerVector idx2lens = sch["idx2lens"];
+    
+    R_xlen_t niter = partitions.length();
+    R_xlen_t idx1len = Rf_xlength(idx1);
+    
+    int64_t* idx1rangeptr = (int64_t*) REAL(idx1range);
+    int64_t idx1_start = *idx1rangeptr, idx1_end = *(idx1rangeptr + 1);
+    
+    if( idx1_start == NA_INTEGER64 || idx1_end < 0 || idx1_start < 0 ){
+        // idx1 are all NAs, no need to subset, return NA
+        
+        T* retptr = ret_ptr;
+        for(R_xlen_t jj = 0; jj < retlen; jj++){
+            *retptr++ = na;
+        }
+        return(false);
+    }
+    
+    int err = -1;
+    std::string error_msg = "";
+    // char* buffer[nbuffers];
+    
+    int ncores = getThreads();
+    if(ncores > niter){
+        ncores = niter;
+    }
+    const boost::interprocess::mode_t mode = boost::interprocess::read_only;
+    const int elem_size = sizeof(B);
+    const bool swap_endian = !isLittleEndian();
+    
+#pragma omp parallel num_threads(ncores) 
+{
+#pragma omp for schedule(static, 1) nowait
+    for(R_xlen_t ii = 0; ii < niter; ii++){
+        
+        int part = partitions[ii];
+        int64_t skips = 0;
+        if(ii > 0){
+            skips = idx2lens[ii - 1];
+        }
+        int64_t idx2len = idx2lens[ii] - skips;
+        
+        // TODO: change
+        T* retptr = ret_ptr + skips * idx1len;
+        for(R_xlen_t jj = 0; jj < idx2len * idx1len; jj++, retptr++ ){
+            *retptr = na;
+        }
+        
+        // TODO: change
+        retptr = ret_ptr + skips * idx1len;
+        
+        SEXP idx2 = idx2s[ii];
+        int64_t idx2_start = NA_INTEGER64, idx2_end = -1;
+        int64_t* ptr2 = (int64_t*) REAL(idx1); 
+        for(ptr2 = (int64_t*) REAL(idx2); idx2len > 0; idx2len--, ptr2++ ){
+            if( *ptr2 == NA_INTEGER64 ){
+                continue;
+            }
+            if( *ptr2 < idx2_start || idx2_start == NA_INTEGER64 ){
+                idx2_start = *ptr2;
+            }
+            if( idx2_end < *ptr2 ){
+                idx2_end = *ptr2;
+            }
+        }
+        
+        if( idx2_start == NA_INTEGER64 || idx2_end < 0 || idx2_start < 0 ){
+            // This is NA partition, no need to subset
+            continue;
+        }
+        
+        // const int idx2_sorted = kinda_sorted(idx2, idx2_start, 1);
+        std::string file = filebase + std::to_string(part) + ".farr";
+        
+        try{
+            boost::interprocess::file_mapping fm(file.c_str(), mode);
+            boost::interprocess::mapped_region region(
+                    fm, mode, 
+                    FARR_HEADER_LENGTH + elem_size * (
+                            idx2_start * block_size + idx1_start
+                    ), 
+                    elem_size * (
+                            idx1_end - idx1_start + 1 +
+                                block_size * (idx2_end - idx2_start)
+                    ));
+            // region.advise(boost::interprocess::mapped_region::advice_sequential);
+            const B* mmap_ptr = static_cast<const B*>(region.get_address());
+            // const int64_t content_size = region.get_size() / elem_size;
+            
+            
+            // prepare for all the pointers, local variables
+            int64_t* idx2ptr = INTEGER64(idx2);
+            R_xlen_t idx2len = Rf_xlength(idx2);
+            R_xlen_t ii_idx2 = 0;
+            int64_t start_idx = idx1_start;
+            
+            int64_t* idx1ptr = INTEGER64(idx1);
+            
+            R_xlen_t jj = 0;
+            T* retptr2 = retptr;
+            
+            for(; ii_idx2 < idx2len; ii_idx2++, idx2ptr++) {
+                
+                if ( *idx2ptr == NA_INTEGER64 ){
+                    continue;
+                }
+                
+                // Rcout << block << "\n";
+                
+                // read current block!
+                retptr2 = retptr + ii_idx2 * idx1len;
+                start_idx = block_size * (*idx2ptr - idx2_start);
+                
+                idx1ptr = INTEGER64(idx1);
+                start_idx -= idx1_start;
+                
+                for(jj = 0; jj < idx1len; jj++, idx1ptr++, retptr2++) {
+                    if(*idx1ptr != NA_INTEGER64){
+                        transform(mmap_ptr + (start_idx + *idx1ptr), retptr2, swap_endian);
+                    }
+                    
+                }
+                
+            }
+        } catch(...) {
+            // Debug use
+            // err = part;
+        }
+        
+    }
+}
+
+    if( err >= 0 ){
+        stop("Error while trying to read partition "+std::to_string(err+1)+
+            ". Reason: " + error_msg);
+        // Rcout << error_msg << "\n";
+    }
+    return(true);
+}
+
+template <typename T, typename B>
+bool FARR_subset_fread(
+        const std::string& filebase, 
+        const List& sch,
+        T* ret_ptr, const T na, const R_xlen_t& retlen,
+        const std::vector<B*> buf_ptrs,
+        void (*transform)(const B*, T*, const bool&)
+){
+    
+    SEXP idx1 = sch["idx1"];
+    SEXP idx1range = sch["idx1range"];
+    List idx2s = sch["idx2s"];
+    int64_t block_size = (int64_t) (sch["block_size"]);
+    IntegerVector partitions = sch["partitions"];
+    IntegerVector idx2lens = sch["idx2lens"];
+    
+    R_xlen_t niter = partitions.length();
+    R_xlen_t idx1len = Rf_xlength(idx1);
+    
+    int64_t* idx1rangeptr = (int64_t*) REAL(idx1range);
+    int64_t idx1_start = *idx1rangeptr, idx1_end = *(idx1rangeptr + 1);
+    
+    if( idx1_start == NA_INTEGER64 || idx1_end < 0 || idx1_start < 0 ){
+        // idx1 are all NAs, no need to subset, return NA
+        
+        T* retptr = ret_ptr;
+        for(R_xlen_t jj = 0; jj < retlen; jj++){
+            *retptr++ = na;
+        }
+        return(false);
+    }
+    
+    int err = -1;
+    std::string error_msg = "";
+    // char* buffer[nbuffers];
+    
+    int ncores = getThreads();
+    if(ncores > niter){
+        ncores = niter;
+    }
+    // const boost::interprocess::mode_t mode = boost::interprocess::read_only;
+    const int elem_size = sizeof(B);
+    // const bool swap_endian = !isLittleEndian();
+    
+#pragma omp parallel num_threads(ncores) 
+{
+#pragma omp for schedule(static, 1) nowait
+    for(R_xlen_t ii = 0; ii < niter; ii++){
+        
+        int part = partitions[ii];
+        int64_t skips = 0;
+        if(ii > 0){
+            skips = idx2lens[ii - 1];
+        }
+        int64_t idx2len = idx2lens[ii] - skips;
+        
+        // TODO: change
+        T* retptr = ret_ptr + skips * idx1len;
+        for(R_xlen_t jj = 0; jj < idx2len * idx1len; jj++, retptr++ ){
+            *retptr = na;
+        }
+        
+        // TODO: change
+        retptr = ret_ptr + skips * idx1len;
+        
+        SEXP idx2 = idx2s[ii];
+        int64_t idx2_start = NA_INTEGER64, idx2_end = -1;
+        int64_t* ptr2 = (int64_t*) REAL(idx1); 
+        for(ptr2 = (int64_t*) REAL(idx2); idx2len > 0; idx2len--, ptr2++ ){
+            if( *ptr2 == NA_INTEGER64 ){
+                continue;
+            }
+            if( *ptr2 < idx2_start || idx2_start == NA_INTEGER64 ){
+                idx2_start = *ptr2;
+            }
+            if( idx2_end < *ptr2 ){
+                idx2_end = *ptr2;
+            }
+        }
+        
+        if( idx2_start == NA_INTEGER64 || idx2_end < 0 || idx2_start < 0 ){
+            // This is NA partition, no need to subset
+            continue;
+        }
+        
+        // const int idx2_sorted = kinda_sorted(idx2, idx2_start, 1);
+        std::string file = filebase + std::to_string(part) + ".farr";
+        
+        FILE* conn = fopen(file.c_str(), "rb");
+        if( conn == NULL ){
+            continue;
+        }
+        
+        try{
+            B* buf_ptr = buf_ptrs[ii % ncores];
+            
+            // prepare for all the pointers, local variables
+            int64_t* idx2ptr = INTEGER64(idx2);
+            R_xlen_t idx2len = Rf_xlength(idx2);
+            R_xlen_t ii_idx2 = 0;
+            // int64_t start_idx = idx1_start;
+
+            int64_t* idx1ptr = INTEGER64(idx1);
+            
+            R_xlen_t jj = 0;
+            T* retptr2 = retptr;
+            
+            for(; ii_idx2 < idx2len; ii_idx2++, idx2ptr++) {
+                
+                if ( *idx2ptr == NA_INTEGER64 ){
+                    continue;
+                }
+                
+                // Rcout << block << "\n";
+                
+                // read current block!
+                retptr2 = retptr + ii_idx2 * idx1len;
+                idx1ptr = INTEGER64(idx1);
+                
+                fseek(conn, FARR_HEADER_LENGTH + elem_size * (
+                        *idx2ptr * block_size + idx1_start
+                ), SEEK_SET);
+                lendian_fread(buf_ptr, elem_size, idx1_end - idx1_start + 1, conn);
+                
+                for(jj = 0; jj < idx1len; jj++, idx1ptr++, retptr2++) {
+                    if(*idx1ptr != NA_INTEGER64){
+                        transform(buf_ptr + (*idx1ptr - idx1_start), retptr2, false);
+                    }
+                    
+                }
+                
+            }
+        } catch(...) {
+            // Debug use
+            // err = part;
+        }
+        
+        if( conn != NULL ){
+            fclose(conn);
+        }
+        
+    }
+}
+
+    if( err >= 0 ){
+        stop("Error while trying to read partition "+std::to_string(err+1)+
+            ". Reason: " + error_msg);
+        // Rcout << error_msg << "\n";
+    }
+    return(true);
+}
+
+SEXP FARR_subset(const std::string& filebase, 
+                 const List& sch,
+                 const SEXPTYPE type,
+                 SEXP ret){
+    std::string fbase = correct_filebase(filebase);
+    
+    SEXP idx1 = sch["idx1"];
+    R_xlen_t idx1len = Rf_xlength(idx1);
+    // IntegerVector partitions = sch["partitions"];
+    // IntegerVector idx2lens = sch["idx2lens"];
+    
+    // R_xlen_t niter = partitions.length();
+    
+    // R_xlen_t retlen = idx1len * idx2lens[niter - 1];
+    // 
+    // SEXPTYPE ret_type = array_memory_sxptype(type);
+    // SEXP ret = PROTECT(Rf_allocVector(ret_type, retlen));
+    
+    R_xlen_t retlen = Rf_xlength(ret);
+    SEXP result_dim = sch["result_dim"];
+    Rf_setAttrib(ret, R_DimSymbol, result_dim);
+    
+    
+    int ncores = getThreads();
+    if( ncores < 1 ){
+        stop("Thread number and buffer pool size must be positive.");
+    }
+    
+    bool use_mmap = false;
+    // Allocate buffers
+    SEXPTYPE buffer_type = file_buffer_sxptype(type);
+    SEXP idx1range = sch["idx1range"];
+    int64_t* idx1rangeptr = INTEGER64(idx1range);
+    int64_t idx1_start = *idx1rangeptr, idx1_end = *(idx1rangeptr + 1);
+    int64_t buffer_nelems = idx1_end - idx1_start + 1;
+    if( idx1_end < 0 || idx1_start == NA_INTEGER64 || idx1_start < 0 ){
+        use_mmap = true;
+    } else if ( buffer_nelems > 2 * idx1len ){ // TODO: test the ratio
+        use_mmap = true;
+    }
+    
+    
+    if(use_mmap){
+        switch(type){
+        case INTSXP: {
+            FARR_subset_mmap<int, int>(
+                    fbase, sch, INTEGER(ret), NA_INTEGER, retlen,
+                    &transform_asis);
+            break;
+        }
+        case REALSXP: {
+            FARR_subset_mmap<double, double>(
+                    fbase, sch, REAL(ret), NA_REAL, retlen,
+                    &transform_asis);
+            break;
+        }
+        case FLTSXP: {
+            FARR_subset_mmap<double, float>(
+                    fbase, sch, REAL(ret), NA_REAL, retlen,
+                    &transform_float);
+            break;
+        }
+        case RAWSXP: {
+            FARR_subset_mmap<Rbyte, Rbyte>(
+                    fbase, sch, RAW(ret), NA_RBYTE, retlen,
+                    &transform_asis);
+            break;
+        }
+        case LGLSXP: {
+            FARR_subset_mmap<int, Rbyte>(
+                    fbase, sch, LOGICAL(ret), NA_LOGICAL, retlen,
+                    &transform_logical);
+            break;
+        }
+        case CPLXSXP: {
+            na_cplx_dbl();
+            Rcomplex na_cplx;
+            na_cplx.i = NA_REAL;
+            na_cplx.r = NA_REAL;
+            FARR_subset_mmap<Rcomplex, double>(
+                    fbase, sch, COMPLEX(ret), na_cplx, retlen,
+                    &transform_complex);
+            break;
+        }
+        default:
+            stop("Unsupported SEXP type");
+        }
+        
+    } else {
+        SEXP buf = PROTECT(Rf_allocVector(buffer_type, ncores * buffer_nelems));
+        switch(type){
+        case INTSXP: {
+            std::vector<int*> buf_ptrs(ncores);
+            for(int i = 0; i < ncores; i++){
+                buf_ptrs[i] = INTEGER(buf) + i * buffer_nelems;
+            }
+            FARR_subset_fread<int, int>(
+                    fbase, sch, INTEGER(ret), NA_INTEGER, retlen,
+                    buf_ptrs,
+                    &transform_asis);
+            break;
+        }
+        case REALSXP: {
+            std::vector<double*> buf_ptrs(ncores);
+            for(int i = 0; i < ncores; i++){
+                buf_ptrs[i] = REAL(buf) + i * buffer_nelems;
+            }
+            FARR_subset_fread<double, double>(
+                    fbase, sch, REAL(ret), NA_REAL, retlen,
+                    buf_ptrs,
+                    &transform_asis);
+            break;
+        }
+        case FLTSXP: {
+            std::vector<float*> buf_ptrs(ncores);
+            for(int i = 0; i < ncores; i++){
+                buf_ptrs[i] = FLOAT(buf) + i * buffer_nelems;
+            }
+            FARR_subset_fread<double, float>(
+                    fbase, sch, REAL(ret), NA_REAL, retlen,
+                    buf_ptrs,
+                    &transform_float);
+            break;
+        }
+        case RAWSXP: {
+            std::vector<Rbyte*> buf_ptrs(ncores);
+            for(int i = 0; i < ncores; i++){
+                buf_ptrs[i] = RAW(buf) + i * buffer_nelems;
+            }
+            FARR_subset_fread<Rbyte, Rbyte>(
+                    fbase, sch, RAW(ret), NA_RBYTE, retlen,
+                    buf_ptrs,
+                    &transform_asis);
+            break;
+        }
+        case LGLSXP: {
+            std::vector<Rbyte*> buf_ptrs(ncores);
+            for(int i = 0; i < ncores; i++){
+                buf_ptrs[i] = RAW(buf) + i * buffer_nelems;
+            }
+            FARR_subset_fread<int, Rbyte>(
+                    fbase, sch, LOGICAL(ret), NA_LOGICAL, retlen,
+                    buf_ptrs,
+                    &transform_logical);
+            break;
+        }
+        case CPLXSXP: {
+            na_cplx_dbl();
+            Rcomplex na_cplx;
+            na_cplx.i = NA_REAL;
+            na_cplx.r = NA_REAL;
+            std::vector<double*> buf_ptrs(ncores);
+            for(int i = 0; i < ncores; i++){
+                buf_ptrs[i] = REAL(buf) + i * buffer_nelems;
+            }
+            FARR_subset_fread<Rcomplex, double>(
+                    fbase, sch, COMPLEX(ret), na_cplx, retlen,
+                    buf_ptrs,
+                    &transform_complex);
+            break;
+        }
+        default:
+            UNPROTECT(1);
+            stop("Unsupported SEXP type");
+        }
+        
+        UNPROTECT(1);
+    }
+    
+    
+    return(ret);
+}
 
 // [[Rcpp::export]]
 SEXP FARR_subset2(
@@ -419,7 +660,7 @@ SEXP FARR_subset2(
         const SEXP reshape = R_NilValue,
         const bool drop = false,
         const bool use_dimnames = true,
-        const size_t thread_buffer = 2097152,
+        size_t thread_buffer = 0,
         int split_dim = 0,
         const int strict = 1
 ) {
@@ -432,6 +673,11 @@ SEXP FARR_subset2(
     
     R_len_t ndims = Rf_length(dim);
     
+    int current_bufsize = get_buffer_size();
+    if( thread_buffer <= 0 ){
+        thread_buffer = current_bufsize;
+    }
+    
     // calculate split_dim
     if( split_dim == NA_INTEGER || split_dim == 0 ){
         split_dim = guess_splitdim(dim, elem_size, thread_buffer);
@@ -471,6 +717,8 @@ SEXP FARR_subset2(
     reshape_or_drop(res, reshape, drop);
     // R_gc();
     
+    set_buffer_size(current_bufsize);
+    
     UNPROTECT(2);
     return(res);
 }
@@ -481,12 +729,12 @@ SEXP FARR_subset2(
 loadNamespace('bit64')
 
 set.seed(1); file <- tempfile(); unlink(file, recursive = TRUE)
-x <- filearray_create(file, 3:5, partition_size = 2, type = "float")
+x <- filearray_create(file, 3:5, partition_size = 1, type = "double")
 x[] <- 1:60
 
-FARR_subset(x$.filebase, x$sexp_type(), list(),
-            dim(x), x$.partition_info[,3], 2,
-            NULL, FALSE, 1, NULL)
+filearray_threads(1)
+FARR_subset2(filebase = x$.filebase, listOrEnv = list(), 
+            reshape = NULL, drop = FALSE)
 
 
 # set_buffer_size(31)
