@@ -1,3 +1,4 @@
+
 require(testthat)
 as_int64 <- function(x){
     realToInt64(x, NA_real_, NA_real_, 0L)
@@ -66,7 +67,7 @@ test_that("C++: IO - subset/assign", {
     file <- tempfile()
     unlink(file, recursive = TRUE)
     dim <- 33:35
-    x <- filearray_create(file, dim, partition_size = 2)
+    x <- filearray_create(file, dim, partition_size = 2, initialize = FALSE)
     
     expect_equal(x[[2]], x$.na)
     
@@ -121,7 +122,7 @@ test_that("C++: IO - subset/assign", {
     expect_error(x[c(rep(c(TRUE,TRUE,TRUE, TRUE),8), TRUE, NA),,])
     
     unlink(file, recursive = TRUE)
-    x <- filearray_create(file, dim, partition_size = 2)
+    x <- filearray_create(file, dim, partition_size = 2, initialize = FALSE)
     y <- array(x$.na, dim)
     x[33:1, , c(35,2,1,3,4,5:34)] <- 1:prod(dim)
     y[33:1, , c(35,2,1,3,4,5:34)] <- 1:prod(dim)
@@ -147,7 +148,7 @@ test_that("C++: IO - subset/assign - complex", {
     file <- tempfile()
     unlink(file, recursive = TRUE)
     dim <- 33:35
-    x <- filearray_create(file, dim, partition_size = 2, type = "complex")
+    x <- filearray_create(file, dim, partition_size = 2, type = "complex", initialize = FALSE)
     
     expect_equal(x[[2]], x$.na)
     
@@ -217,7 +218,7 @@ test_that("C++: IO - subset/assign - complex", {
     expect_error(x[c(rep(c(TRUE,TRUE,TRUE, TRUE),8), TRUE, NA),,])
     
     unlink(file, recursive = TRUE)
-    x <- filearray_create(file, dim, partition_size = 2, type = 'complex')
+    x <- filearray_create(file, dim, partition_size = 2, type = 'complex', initialize = FALSE)
     y <- array(x$.na, dim)
     x[33:1, , c(35,2,1,3,4,5:34)] <- tmp
     y[33:1, , c(35,2,1,3,4,5:34)] <- tmp
@@ -243,7 +244,7 @@ test_that("C++: IO - subset/assign - float", {
     file <- tempfile()
     unlink(file, recursive = TRUE)
     dim <- 33:35
-    x <- filearray_create(file, dim, partition_size = 2, type = "float")
+    x <- filearray_create(file, dim, partition_size = 2, type = "float", initialize = FALSE)
     
     expect_equal(x[[2]], x$.na)
     
@@ -318,7 +319,7 @@ test_that("C++: IO - subset/assign - float", {
     expect_error(x[c(rep(c(TRUE,TRUE,TRUE, TRUE),8), TRUE, NA),,])
     
     unlink(file, recursive = TRUE)
-    x <- filearray_create(file, dim, partition_size = 2)
+    x <- filearray_create(file, dim, partition_size = 2, initialize = FALSE)
     y <- array(x$.na, dim)
     x[33:1, , c(35,2,1,3,4,5:34)] <- 1:prod(dim)
     y[33:1, , c(35,2,1,3,4,5:34)] <- 1:prod(dim)
@@ -337,7 +338,7 @@ test_that("C++: IO - type conversion", {
     file <- tempfile()
     unlink(file, recursive = TRUE)
     dim <- 3:5
-    x <- filearray_create(file, dim, type = 'integer', partition_size = 2)
+    x <- filearray_create(file, dim, type = 'integer', partition_size = 2, initialize = FALSE)
     
     expect_equal(x[[2]], x$.na)
     
@@ -350,7 +351,7 @@ test_that("C++: IO - type conversion", {
     
     
     unlink(file, recursive = TRUE)
-    x <- filearray_create(file, dim, type = 'logical', partition_size = 2)
+    x <- filearray_create(file, dim, type = 'logical', partition_size = 2, initialize = FALSE)
     y <- array(x$.na, dim)
     tmp <- sample(c(1.1, 0, NA), replace = TRUE, size = 60)
     x[] <- tmp
@@ -358,7 +359,7 @@ test_that("C++: IO - type conversion", {
     expect_equal(x[], y)
     
     unlink(file, recursive = TRUE)
-    x <- filearray_create(file, dim, type = 'raw')
+    x <- filearray_create(file, dim, type = 'raw', initialize = FALSE)
     y <- array(x$.na, dim)
     tmp <- sample(0:255, replace = TRUE, size = 60)
     tmp[sample(1:60, size = 10)] <- NA
